<?php

namespace Shopbydedi\Shipping;

use SiteSQL;

class MondialRelay implements RelayInterface {

    const STAT_COLIS_TRAITEMENT = 81;
    const STAT_COLIS_LIVRE = 82;

    public $sql;
    public $cle_prive;
    public $enseigne;
    public $enseigne_url;
    public $expediteur;
    public $format;
    private $wsdl;

    function __construct($config) {
        $this->sql = SiteSQL::getInstance();

        $this->wsdl = 'http://api.mondialrelay.com/webservice.asmx?WSDL';

        $this->cle_prive = $config['mondial_relay']['cle_prive'];
        $this->enseigne = $config['mondial_relay']['enseigne'];
        $this->enseigne_url = $config['mondial_relay']['enseigne_url'];

        $this->expediteur = array();
        $this->expediteur['addr1'] = $config['mondial_relay']['addr1'];
        $this->expediteur['addr3'] = $config['mondial_relay']['addr1'];
        $this->expediteur['ville'] = $config['mondial_relay']['ville'];
        $this->expediteur['cp'] = $config['mondial_relay']['cp'];
        $this->expediteur['pays'] = 'FR';
        $this->expediteur['tel'] = $config['mondial_relay']['tel'];

        $this->format = 'A6';  // valeur possible : A4, A5, A6
    }

    /**
     * @return bool
     */
    static public function checkConnection() {
        $fp = @fsockopen("www.mondialrelay.fr", 80, $errno, $errstr, 3);
        return (!$fp) ? false : true;
    }

    /**
     * @param array $formvars
     * @return Relay[]
     */
    public function getList($formvars = array())
    {

        if (empty($formvars['ville']))
            $formvars['ville'] = '';
        if (empty($formvars['code_postal']))
            $formvars['code_postal'] = '';
        if (empty($formvars['poids']))
            $formvars['poids'] = '';

        if (empty($formvars['pays']))
            $formvars['pays'] = 'FR';

        $param = array(
            'Enseigne' => $this->enseigne,
            'Pays' => $formvars['pays'],
            'Ville' => $formvars['ville'],
            'CP' => $formvars['code_postal'],
            'Poids' => $formvars['poids'],
        );

        return $this->WSI3_PointRelais_Recherche($param);
    }

    /**
     * @param string $relayId
     * @param string $countryIso
     *
     * @return Relay
     */
    public function getRelay($relayId, $countryIso = '')
    {

        if (empty($countryIso)) {
            $countryIso = 'FR';
        }

        $param = array(
            'Enseigne' => $this->enseigne,
            'Pays' => $countryIso,
            'NumPointRelais' => $relayId,
        );

        return $this->WSI3_PointRelais_Recherche($param);
    }

    /**
     * @param array $data
     * @return Relay
     */
    public function formatRelay(array $data)
    {
        $relay = new Relay();
        $relay->id = trim($data['Num']);
        $relay->name = trim($data['LgAdr1']);
        $relay->addr1 = trim($data['LgAdr3']);
        $relay->addr2 = trim($data['LgAdr2']);
        $relay->zipcode = trim($data['CP']);
        $relay->city = trim($data['Ville']);
        $relay->country = trim($data['Pays']);
        $relay->latitude = $data['Latitude'];
        $relay->longitude = $data['Longitude'];
        $relay->distance = $data['Distance'];

        $horaires = array('Lundi' => array(), 'Mardi' => array(), 'Mercredi' => array(), 'Jeudi' => array(), 'Vendredi' => array(), 'Samedi' => array(), 'Dimanche' => array());
        foreach ($horaires as $key => $hor) {
            foreach ($data['Horaires_' . $key]->string as $k => $horMr) {
                $res = $this->parseHoraire($horMr);
                if (!empty($res)) {
                    $horaires[$key][$k] = $res;
                }
            }
        }

        $relay->horaires = $horaires;

        return $relay;
    }

    protected function parseHoraire($plageHoraire)
    {
        $result = '';
        if ($plageHoraire != '0000') {
            $result = substr_replace($plageHoraire, ':', 2, 0);
        }

        return $result;
    }

    /**
     * @param $param
     *
     * @return Relay[]|Relay
     */
    protected function WSI3_PointRelais_Recherche($param)
    {
        // generation de la clé de securité
        $param['Security'] = $this->generateSecurity($param);

        // envoie de la demande a Mondial Relay
        $soap = new \SoapClient($this->wsdl);
        $result = $soap->WSI3_PointRelais_Recherche($param);

        // traitement resultat pour avoir une liste de relay / un point relay
        if ($result->WSI3_PointRelais_RechercheResult->STAT == 0) {
            if (is_array($result->WSI3_PointRelais_RechercheResult->PointsRelais->PointRelais_Details)) {
                $relais = array();
                foreach($result->WSI3_PointRelais_RechercheResult->PointsRelais->PointRelais_Details as $key => $value) {
                    $relais[] = $this->formatRelay((array) $value);
                }

                return $relais;
            } else {
                return $this->formatRelay((array) $result->WSI3_PointRelais_RechercheResult->PointsRelais->PointRelais_Details);
            }
        }

        return $result->WSI3_PointRelais_RechercheResult;
    }

    /**
     * @param $param
     * @return string
     */
    protected function generateSecurity($param)
    {
        $security = '';
        foreach ($param as $p) {
            $security.= $p;
        }
        return strtoupper(md5($security . $this->cle_prive));
    }


    /**
     *
     * /!\ ATTENTION /!\
     *
     * Tout est a revoir en dessous
     *
     */

    public function getInfosColis($formvars = array()){

        $soap = new \SoapClient($this->wsdl);

        $param = array(
            'Enseigne' => $this->enseigne,
            'Expedition' => $formvars['numero_colis'],
            'Langue' => 'FR',
            'Security' => strtoupper(md5($this->enseigne . $formvars['numero_colis'] . 'FR' . $this->cle_prive)),
        );

        $rep = $soap->WSI2_TracingColisDetaille($param);

        return $rep->WSI2_TracingColisDetailleResult;
    }

    /**
     * TODO : A revoir
     */
    public function updateStatutLivraison(){
        $commande = new \Commande;

        $param = array();
        $param['list_commande_statut_id'] = Commande::STATUT_EXPEDIE . ', ' . Commande::STATUT_DISPO_RELAIS;
        $param['list_commande_livraison_code'] = "'24R'";
        $param['withTracking'] = true;

        // recup des commandes mondial relay expedié, ou livré en point relais
        $arrCmd = $commande->get($param);

        foreach($arrCmd as $cmd){
            $infosLivr = $this->getInfosColis(array('numero_colis' => $cmd['commande_livraison_colis']));
            if($infosLivr->STAT == self::STAT_COLIS_LIVRE){
                // change statut livré
                // echo 'commande #' . $cmd['commande_id'] . ' livré<br />';
                $commande->changeStatut($cmd['commande_id'], STATUT_LIVRE);
            }elseif($infosLivr->STAT == self::STAT_COLIS_TRAITEMENT && $cmd['commande_statut_id'] == STATUT_EXPEDIE){
                foreach($infosLivr->Tracing->ret_WSI2_sub_TracingColisDetaille as $il){
                    if($il->Libelle == 'DISPONIBLE AU POINT RELAIS'){
                        // change statut en point relais
                        // echo 'commande #' . $cmd['commande_id'] . ' dispo point relais<br />';
                        $commande->changeStatut($cmd['commande_id'], STATUT_DISPO_RELAIS, true);
                        break;
                    }
                }
            }
        }
    }

    public function haveEtiquette($idCommande) {
        $commande = new Commande();
        $c = $commande->get(array('commande_id' => $idCommande));

        if (empty($c))
            return 'no-commande';
        else
            $c = $c[0];

        // verif commande mondial relay
        if ($c['commande_livraison_code'] != '24R')
            return 'commande-no-mr';

        // verif en cours de prepa
        /*if ($c['commande_statut_id'] != 3)
            return 'commande-no-prepa';*/

        return (!empty($c['commande_livraison_colis']));
    }

    public function getEtiquette($idCommande, $c = array(), $new = false) {

        $commande = new Commande();
        if(empty($c)){
            $c = $commande->get(array('commande_id' => $idCommande));
        }

        if (empty($c))
            return 'no-commande';
        else
            $c = $c[0];

        // verif commande mondial relay
        if ($c['commande_livraison_code'] != '24R')
            return 'commande-no-mr';

        // verif en cours de prepa
        $arrStatutOK = array(
            Commande::STATUT_PREPARATION,
            Commande::STATUT_EXPEDIE,
            Commande::STATUT_LIVRE,
            Commande::STATUT_DISPO_RELAIS
        );

        if (!in_array($c['commande_statut_id'] , $arrStatutOK)) {
            return 'commande-no-prepa';
        }

        if (empty($c['commande_livraison_colis']) || $new == true) {
            $c['commande_livraison_colis'] = $this->generateEtiquette($c);
            if ($c['commande_livraison_colis'] == false)
                return 'erreur-generation-etiquette';
        }

        $param = array(
            'Enseigne' => $this->enseigne,
            'Expeditions' => $c['commande_livraison_colis'],
            'Langue' => 'FR'
        );

        $cle = '';
        foreach ($param as $p) {
            $cle .= $p;
        }

        $param['Security'] = strtoupper(md5($cle . $this->cle_prive));

        $soap = new SoapClient($this->wsdl);
        $rep = $soap->WSI2_GetEtiquettes($param);

        $return = $rep->WSI2_GetEtiquettesResult;

        if ($return->STAT == 0) {
            switch ($this->format) {
                case 'A4':
                    $urlPDF = $return->URL_PDF_A4;
                    break;

                case 'A5':
                    $urlPDF = $return->URL_PDF_A5;
                    break;

                case 'A6':
                    $urlPDF = str_replace('format=A4', 'format=10x15', $return->URL_PDF_A4);
                    break;

                default:
                    $urlPDF = $return->URL_PDF_A4;
                    break;
            }
            return 'http://www.mondialrelay.com' . $urlPDF;
        } else {
            /* var_dump($return); */
            return 'erreur-recuperation-etiquette';
        }
    }

    function recupAllEtiquette() {

        $commande = new Commande();
        $param = array();
        $param['list_commande_livraison_code'] = "'24R'";
        $param['commande_statut_id'] = STATUT_PREPARATION;
        $arrCmd = $commande->get($param);
        $arrExpe = array();
        foreach($arrCmd as $c) {
            if(!empty($c['commande_livraison_colis']))
                array_push($arrExpe,$c['commande_livraison_colis']);
        }

        $out = "";
        if(!empty($arrExpe)) {

            $pdf = $this->recupPDF($arrExpe);

            //if(isset($pdf['URL_PDF_A4'])) $out .= '<a href="'.$pdf['URL_PDF_A4'].'">Etiquettes A4</a><br/>';
            //if(isset($pdf['URL_PDF_A5'])) $out .= '<a href="'.$pdf['URL_PDF_A5'].'">Etiquettes A5</a><br/>';
            if(isset($pdf['URL_PDF_A6'])) $out .= '<a href="'.$pdf['URL_PDF_A6'].'">Etiquettes A6</a><br/>';

        }
        else {
            $out .= "Aucune etiquette a imprimer !";
        }

        return $out;

    }

    function recupPDF($arrExpe) {

        if(empty($arrExpe)) return false;

        $listExpe = implode(';',$arrExpe);

        $param = array(
            'Enseigne' => $this->enseigne,
            'Expeditions' => $listExpe,
            'Langue' => 'FR'
        );

        $cle = '';
        foreach($param as $p) {
            $cle .= $p;
        }

        $param['Security'] = strtoupper(md5($cle.$this->cle_prive));

        $soap = new SoapClient($this->wsdl);
        $rep = $soap->WSI2_GetEtiquettes($param);

        $return = $rep->WSI2_GetEtiquettesResult;

        if($return->STAT==0) {

            return array(
                'URL_PDF_A4'=>'http://www.mondialrelay.com/'.$return->URL_PDF_A4,
                'URL_PDF_A5'=>'http://www.mondialrelay.com/'.$return->URL_PDF_A5,
                'URL_PDF_A6'=>'http://www.mondialrelay.com/'.str_replace('format=A4', 'format=10x15', $return->URL_PDF_A4)
            );

        }
        else {
            var_dump($return);
            return false;
        }
    }

    private function generateEtiquette($c) {

        // spec
        // $c['commande_poids']=600;
        // on enleve la civilite du nom
        $infosNom = explode(' ', $c['facturation_nom']);
        unset($infosNom[0]);
        $c['facturation_nom'] = implode(' ', $infosNom);
        //--

        // on enleve les slashs
        /*$arrIndexSlash = array('facturation_nom', 'facturation_rue2', 'facturation_rue', 'facturation_ville');
        foreach($arrIndexSlash as $iSlash){
            if(!empty($c[$iSlash])){
                $c[$iSlash] = stripslashes($c[$iSlash]);
            }
        }*/

        if ($c['commande_poids'] < 100)
            $c['commande_poids'] = 100;
        $param = array(
            'Enseigne' => $this->enseigne,
            'ModeCol' => 'CCC',
            'ModeLiv' => $c['commande_livraison_code'],
            'NDossier' => $c['commande_id'],
            'NClient' => $c['client_id'],
            'Expe_Langage' => 'FR',
            'Expe_Ad1' => $this->expediteur['addr1'],
            //'Expe_Ad2' => ,
            'Expe_Ad3' => $this->expediteur['addr3'],
            //'Expe_Ad4' => ,
            'Expe_Ville' => $this->expediteur['ville'],
            'Expe_CP' => $this->expediteur['cp'],
            'Expe_Pays' => $this->expediteur['pays'],
            'Expe_Tel1' => $this->expediteur['tel'],
            //'Expe_Tel2' => ,
            //'Expe_Mail' => ,
            'Dest_Langage' => 'FR',
            'Dest_Ad1' => $this->parse(utf8_decode(substr($c['facturation_nom'], 0, 31))),
            'Dest_Ad2' => $this->parse(utf8_decode(substr($c['facturation_rue2'], 0, 31))),
            'Dest_Ad3' => $this->parse(utf8_decode(substr($c['facturation_rue'], 0, 31))),
            //'Dest_Ad4' => ,
            'Dest_Ville' => $this->parse(utf8_decode(substr(preg_replace("#[0-9]#", "", $c['facturation_ville']), 0, 24))),
            'Dest_CP' => trim(str_replace(' ', '', $c['facturation_cp'])),
            'Dest_Pays' => $c['livraison_pays_code'],
            //'Dest_Tel1' => ,
            //'Dest_Tel2' => ,
            'Dest_Mail' => $c['client_email'],
            'Poids' => $c['commande_poids'],
            //'Longueur' => ,
            //'Taille' => ,
            'NbColis' => 1,
            'CRT_Valeur' => 0,
            'CRT_Devise' => 'EUR',
            'Exp_Valeur' => ($c['montant_paye'] * 100),
            'Exp_Devise' => 'EUR',
            //'COL_Rel_Pays' => ,
            //'COL_Rel' => ,
            'LIV_Rel_Pays' => $c['livraison_pays_code'],
            'LIV_Rel' => str_pad($c['relais_mondial_relay'], 6, "0", STR_PAD_LEFT),
            'TAvisage' => 'N',
            'TReprise' => 'N',
            'Montage' => 0,
            'TRDV' => 'N',
            'Assurance' => 0,
            'Instructions' => '',
            //'Security' => ,
            //'Texte' => '',
        );

        $cle = '';
        foreach ($param as $p) {
            $cle .= $p;
        }

        $param['Security'] = strtoupper(md5($cle . $this->cle_prive));

        $soap = new SoapClient($this->wsdl);
        $rep = $soap->WSI2_CreationEtiquette($param);

        $return = $rep->WSI2_CreationEtiquetteResult;

        if ($return->STAT == 0) {
            if(empty($c['commande_livraison_colis'])){
                // si 1ere etiquette, on insere infos au niveau de la commande
                $sql = "UPDATE commande SET commande_livraison_colis = '" . $return->ExpeditionNum . "' WHERE commande_id = '" . $c['commande_id'] . "'";
            }else{
                // sinon, dans une nouvelle table
                $sql = "INSERT INTO commande_etiquette_mr SET";
                $sql.= " commande_id = '".$c['commande_id']."'";
                $sql.= ", commande_etiquette_mr_date_ajout = NOW()";
                $sql.= ", commande_etiquette_mr_colis = '" . $return->ExpeditionNum . "'";
            }
            $this->sql->query($sql);
            return $return->ExpeditionNum;
        } else {
            echo 'Erreur Commande : ' . $c['commande_id'] . '<br/>';
            echo $this->getErreurRetour($return->STAT);
            /* var_dump($return) . '<br/><br/>';
              print_r($param) . '<br/>';
              echo '<br/><br/>'; */
            return false;
        }
    }

    // expedie les commandes qui ont été préparé
    public function expedieCommande() {
        $commande = new Commande;
        $param = array();
        $param['list_commande_livraison_code'] = "'24R'";
        $param['commande_statut_id'] = Commande::STATUT_PREPARATION;
        $param['withTracking'] = true;

        // recup des commandes en cours de prépa, mondial relay et qui ont un numéro de tracking
        $arrCommande = $commande->get($param);

        foreach ($arrCommande as $cmd) {
            $commande->changeStatut($cmd['commande_id'], Commande::STATUT_EXPEDIE, true);  // on ne passe pas le numéro de tracking car deja mis lors de la generation de l'etiquette
        }

        return count($arrCommande);
    }

    public function getUrlEtiquette($trackingColis){
        $crc = md5('<' . $this->enseigne_url . '>' . $trackingColis . '<' . $this->cle_prive . '>');
        $url = 'http://www.mondialrelay.com/PDF/StickerMaker2.aspx?ens='.$this->enseigne_url.'&expedition='.$trackingColis.'&lg=FR&format=10x15&crc='.$crc;
        return $url;
    }

    public function getUrlSuivi($trackingColis) {
        $crc = md5('<' . $this->enseigne_url . '>' . $trackingColis . '<' . $this->cle_prive . '>');
        // $url = 'http://www.mondialrelay.com/public/permanent/tracking2.aspx?ens=' . $this->enseigne_url . '&exp=' . $trackingColis . '&crc=' . $crc;
        $url = 'http://www.mondialrelay.fr/suivi-de-colis/?ens=' . $this->enseigne_url . '&exp=' . $trackingColis . '&crc=' . $crc;
        return $url;
    }

    private function parse($text) {
        $text = trim(mb_strtolower($text));

        $table = array(
            'Š' => 'S', 'š' => 's', 'Ð' => 'D', 'd' => 'd', 'Ž' => 'Z', 'ž' => 'z', 'C' => 'C', 'c' => 'c', 'C' => 'C', 'c' => 'c',
            'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A', 'Æ' => 'A', 'Ç' => 'C', 'È' => 'E', 'É' => 'E',
            'Ê' => 'E', 'Ë' => 'E', 'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'Ñ' => 'N', 'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O',
            'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O', 'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ý' => 'Y', 'Þ' => 'B', 'ß' => 'Ss',
            'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a', 'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c', 'è' => 'e', 'é' => 'e',
            'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i', 'ð' => 'o', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o',
            'ô' => 'o', 'õ' => 'o', 'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ý' => 'y', 'þ' => 'b',
            'ÿ' => 'y', 'R' => 'R', 'r' => 'r', ' - ' => '-', ' ' => '-', '&' => 'et'
        );

        $text = strtr($text, $table);

        $text = preg_replace('/[^a-z0-9-]/', '', $text);

        return $text;
    }

    public function getError($code) {
        if (!empty(self::$arrError[$code]))
            return self::$arrError[$code];
    }

    private function initError() {
        self::$arrError = array(
            'no-commande' => "Aucune commande ne correspond &agrave; ce num&eacute;ro",
            'commande-no-mr' => "Cette commande n'est pas une commande Mondial Relay",
            'commande-no-prepa' => "Cette commande n'est pas en cours de pr&eacute;paration",
            'erreur-generation-etiquette' => "Une erreur c'est produit lors de la g&eacute;n&eacute;ration de l'etiquette. Veuillez recommencer.",
            'erreur-recuperation-etiquette' => "Une erreur c'est produit lors de la r&eacute;cup&eacute;ration de l'etiquette. Veuillez recommencer."
        );
    }

}